/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: chassisd-msg.c,v 1.8 2002/11/19 04:43:32 tonyw Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
#ifdef SSC
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <unistd.h>

#include "nfx-incl.h"
#include "../sm-anpssc/anpssc-api.h"
#include "../sm-chassis/cm-api.h"
#include "../sm-elog/elog-api.h"
#include "../sm-utils/sys-utils-api.h"
#include "../sm-event/event-api.h"
#include "../sm-event/emrs-api.h"
#include "../sm-seep/seep.h"
#include "cm.h"
#ifdef COUGAR
#include "cm-api-cg.h"
#include "../ssc-genlib/cm-util-bc.h"
#endif
#ifdef BOBCAT
#include "cm-api-bc.h"
#include "../ssc-genlib/cm-util-bc.h"
#endif
#include "../ssc-genlib/genlib.h"
#ifdef DMALLOC
#include "dmalloc.h"
#endif

int	chassisd_rxif;
/* Flag counter to indicate heartbeats from toshiba are received */
int cm_heartbeat_rx = 0;

#define CHASSISD_DEBUG_RX_MSG	0
#define CHASSISD_DEBUG_TX_MSG	0

int chassisd_sendMsg(void *, int, char *, uint32,
									uint32, struct timeval*, void *, uint32);
int cm_sendInfoReq(void);
int cm_sendTempReq(int slot, int cpu);

int chassisd_rcvMsg(void);
int chassisd_procRcvMsg(char *, int, uchar8 *, uint32, uint32);
int chassisd_procRebootReq(chassisd_msgDesc_t * mdesc);
int chassisd_procInfoReq(chassisd_msgDesc_t * mdesc);
int chassisd_procVersionReq(chassisd_msgDesc_t * mdesc);
int chassisd_procPowerReq(chassisd_msgDesc_t * mdesc);
int chassisd_procTempReq(chassisd_msgDesc_t * mdesc);
int chassisd_procDebugReq(chassisd_msgDesc_t * mdesc);

int cm_procInfoResp(chassisd_msgDesc_t * mdesc);
int cm_procVersionResp(chassisd_msgDesc_t * mdesc);
int cm_procVersionReq(chassisd_msgDesc_t * mdesc);

int cm_procHeartbeatInfo(chassisd_msgDesc_t * mdesc);
int chassis_send_memoryUpgrade(chassisd_msgDesc_t *mdesc);

int convert_to_magicNo(char* str);

int cm_cardType[3] = { NFX_CARDTYPE_SSC, NFX_CARDTYPE_NFP, NFX_CARDTYPE_FCNIM };

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_msgInit(void)
{
	chassisd_rxif = regAgileApp("cm");
	if (chassisd_rxif < 0) {
		perror("chassisd error registering for messages");
		exit(1);
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_sendMsg(
	void	* msg,
	int		len,
	char	* app,
	uint32	to_cpu,
	uint32	to_slot,
	struct timeval	* to,
	void	* rsp_buf,
	uint32	rsp_len
)
{
	int	rc;

#if CHASSISD_DEBUG_TX_MSG
	printf("CHASSISD tx msg: %d bytes to app: %s, cpu: %d, slot: %d\n",
													len, app, to_cpu, to_slot);
#endif

	rc = sendAgileMsg(	chassisd_rxif,
						(char *)msg,
						len,
						app,
						to_cpu,
						to_slot,
						0,
						to,
						(char *)rsp_buf,
						rsp_len);

        if (rc < 0) {
            E_LOG(class_1, err_s, eee_getApplicationId("cm"), 0, 0, 0,
              ("CM: %s: Error sending agile msg to cpu %d, slot %d, rc %d.\n",
			  __FUNCTION__, to_cpu, to_slot, rc));
        }

        return rc;
}

#if defined(CHEETAH)
/*-----------------------------------------------------------------
 * Name: cm_sendInfoReq()
 *
 * Description: When chassisd starts, send info req message to Toshiba
 *				to retrieve current state of system
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/24/02
 *
 *-----------------------------------------------------------------
 */
int
cm_sendInfoReq()
{
	cmMsgInfoReq_t		req;
	cmMsgInfoResp_t		resp;
	chassisd_msgDesc_t	mdesc;
    struct timeval		to;
	int					rc;

	to.tv_sec = 3;
	to.tv_usec = 0;

	bzero(&resp, sizeof(resp));

	req.hdr.mid = CM_SET_INFO_REQ;
	req.hdr.srcSlot = 0;
	req.hdr.srcCpu = 0;
	req.hdr.srcApp = chassisd_rxif;
	req.hdr.srcPlane = MGMTPLANE;

	rc = chassisd_sendMsg(&req, sizeof(req), "cm",
							1, 0, &to,
							&resp, sizeof(resp));

	if (rc == sizeof(resp)) {
		if (resp.hdr.mid == CM_SET_INFO_RESP) {
			//printf("Got Info Response message\n");
			mdesc.msg = &resp;
			mdesc.len = rc;
			mdesc.srcCpu = 1;
			mdesc.srcSlot = 0;
			cm_procInfoResp(&mdesc);
		}
		else
			printf("Error in response from SET_INFO_REQ, mid = 0x%x\n",
																resp.hdr.mid);
	}

	return NFX_OK;
}
#endif

/*-----------------------------------------------------------------
 * Name: cm_sendVersionReq(slot, cpu)
 *
 * Description: When chassisd starts, send version req messages to all
 *				cpus to get their version strings
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
int
cm_sendVersionReq(
	int		slot,
	int		cpu
)
{
	cmMsgVersionReq_t	req;

	req.hdr.mid = CM_SET_VERSION_REQ;
	req.hdr.srcSlot = 0;
	req.hdr.srcCpu = 0;
	req.hdr.srcApp = chassisd_rxif;
	req.hdr.srcPlane = MGMTPLANE;

	(void) chassisd_sendMsg(&req, sizeof(req), "cm",
							cpu, slot, NULL,
							0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: cm_sendTempReq(slot, cpu)
 *
 * Description: Send temperature req messages to all
 *				cpus to get their temperature
 *
 * Created by:	Tony Wong
 *
 * Date Created:	7/19/03
 *
 *-----------------------------------------------------------------
 */
int
cm_sendTempReq(
	int		slot,
	int		cpu
)
{
	cmMsgTempReq_t req;

	req.hdr.mid = CHASSISD_GET_TEMP_REQ;
	req.hdr.srcSlot = 0;
	req.hdr.srcCpu = 0;
	req.hdr.srcApp = chassisd_rxif;
	req.hdr.srcPlane = MGMTPLANE;

	(void) chassisd_sendMsg(&req, sizeof(req), "cm",
							cpu, slot, NULL,
							0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: chassisd_rcvMsg()
 *
 * Description: Registered receive function for "cm" application on BSD
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_rcvMsg()
{
	int		bytes;
	uint32	srcCpu, srcSlot, srcPlane;
	char	srcApp[32];
	char	msg[2000];
    struct timeval to = {1,0};

	bytes = recvAgileMsg(	chassisd_rxif,
							&msg,
							sizeof(msg),
							&to,
							srcApp,
							&srcCpu,
							&srcSlot,
							&srcPlane);

	if (bytes < 0)
		return 0;
	else if (bytes == 0)
		return 0;
	else {
#if CHASSISD_DEBUG_RX_MSG
		printf("CHASSISD rx msg: %d bytes from app: %s, cpu: %d, slot: %d\n",
											bytes, srcApp, srcCpu, srcSlot);
#endif
		chassisd_procRcvMsg(msg, bytes, srcApp, srcCpu, srcSlot);
	}

	return 0;
}

/*-----------------------------------------------------------------
 * Name: chassisd_procRcvMsg()
 *
 * Description: process the message received by chassisd
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procRcvMsg(
	char	* msg,
	int		mlen,
	uchar8	* srcApp,
	uint32	srcCpu,
	uint32	srcSlot
)
{
	chassisd_msg_hdr_t	* hdr;
	uint32				mid;
	chassisd_msgDesc_t	mdesc;
	int slot, cpu;

	hdr = (chassisd_msg_hdr_t *)msg;

	mdesc.msg = msg;
	mdesc.len = mlen;
	mdesc.srcCpu = srcCpu;
	mdesc.srcSlot = srcSlot;
	memcpy(mdesc.srcApp, srcApp, 32);

	mid = hdr->mid;

	switch(mid) {
		case CHASSISD_REBOOT_REQ:
			if (mlen < sizeof(chassisdMsgRebootReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, REBOOT_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procRebootReq(&mdesc);
			break;

		case CHASSISD_GET_CHASSIS_INFO_REQ:
			if (mlen < sizeof(chassisdMsgInfoReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, CHASSIS_INFO_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procInfoReq(&mdesc);
			break;

		case CHASSISD_GET_VERSION_REQ:
#if defined(BOBCAT) || defined(COUGAR)
		    for (slot=1; slot<CM_NUM_SLOTS; slot++)
#else
		    for (slot=0; slot<CM_NUM_SLOTS; slot++)
#endif
            {
		        for (cpu=0; cpu<CM_NUM_CPUS; cpu++) {
				if((slot == 0) && (cpu == 0))
				{
					/* do nothing */
				}	
				else
		           		cm_sendVersionReq(slot, cpu);
		        }
		    }

			if (mlen < sizeof(chassisdMsgVersionReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, GET_VERSION_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procVersionReq(&mdesc);
			break;

		case CHASSISD_GET_POWER_REQ:
			if (mlen < sizeof(chassisdMsgPowerReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, GET_POWER_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procPowerReq(&mdesc);
			break;

		case CHASSISD_GET_TEMP_REQ:
#if defined(BOBCAT) || defined(COUGAR)
        {
            uint32 temp;

            if (cm_read_temperature_bc(&temp) == NFX_OK) {
                cm.slot[0].temp_valid = 1;
                cm.slot[0].temp = temp;
            }
            chassisd_procTempReq(&mdesc);
        }
#else
		    for (slot=0; slot<CM_NUM_SLOTS; slot++) {
		        for (cpu=0; cpu<CM_NUM_CPUS; cpu++) {
		            cm_sendTempReq(slot, cpu);
		        }
		    }
			if (mlen < sizeof(chassisdMsgTempReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, GET_TEMP_REQ, mlen = %d\n", mlen);
#endif
			}
			else {
				cm_sendInfoReq();
				chassisd_procTempReq(&mdesc);
			}
#endif
			break;

		case CHASSISD_DEBUG_REQ:
			if (mlen < sizeof(chassisdMsgDebugReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, DEBUG_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				chassisd_procDebugReq(&mdesc);
			break;

		case CM_SET_VERSION_REQ:
			if (mlen < sizeof(cmMsgVersionReq_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, CM_SET_VERSION_REQ, mlen = %d\n", mlen);
#endif
			}
			else
				cm_procVersionReq(&mdesc);
			break;

		case CM_SET_INFO_RESP:
			if (mlen < sizeof(cmMsgInfoResp_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, CM_SET_INFO_RESP, mlen = %d\n", mlen);
#endif
			}
			else
				cm_procInfoResp(&mdesc);
			break;

		case CM_SET_VERSION_RESP:
			if (mlen < sizeof(cmMsgVersionResp_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, CM_SET_VERSION_RESP, mlen = %d\n", mlen);
#endif
			}
			else
				cm_procVersionResp(&mdesc);
			break;

		case CM_SEND_HEARTBEAT_INFO:
			if (mlen < sizeof(cmHeartbeatMsgInfo_t)) {
#if CHASSISD_DEBUG_RX_MSG
				printf("msg too small, CM_SEND_HEARTBEAT_INFO, mlen = %d\n", mlen);
#endif
			}
			else {
				cm_procHeartbeatInfo(&mdesc);
			}	
			break;

		case CHASSISD_GET_SERIALNUM:
			if (mlen < sizeof(chassisdMsgMemUpgradeReq_t)) {
				printf("msg too small, CHASSISD_GET_SERIALNUM, mlen = %d\n", mlen);
			}	
			else	
				chassis_send_memoryUpgrade(&mdesc);
			break;

#if defined(BOBCAT)
		case CM_GET_CHASSIS_SERIAL:
        {
            seepPCB_t seep;
            cmMsgSerialNoResp_t resp;

            bzero(&resp, sizeof(cmMsgSerialNoResp_t));
            resp.hdr.srcSlot = 0;
            resp.hdr.srcCpu = 0;
            resp.hdr.srcApp = chassisd_rxif;
            resp.hdr.srcPlane = EEE_MGMT;
            if (cm_read_seep_bc((void *)&seep,0,sizeof(seepPCB_t)) == NFX_OK) {
                strncpy(resp.serialNo, seep.serial_num, SEEP_CHASSIS_SERIAL_LEN);
                resp.serialNo[SEEP_CHASSIS_SERIAL_LEN-1] = '\0';
            } 
            else {
                strcpy(resp.serialNo, "0000000000");
            }
            resp.hdr.mid = CM_GET_CHASSIS_SERIAL;
            chassisd_sendMsg(&resp, sizeof(resp), srcApp, srcCpu, srcSlot,
                             NULL, 0, 0);
            break;
        }
#endif
#if defined(COUGAR)
		case CM_GET_CHASSIS_SERIAL:
        {
            cmMsgSerialNoResp_t resp;

            if (chsSeep.serial_num != '\0') {
                strncpy(resp.serialNo, chsSeep.serial_num,
                        SEEP_CHASSIS_SERIAL_LEN);
                resp.serialNo[SEEP_CHASSIS_SERIAL_LEN-1] = '\0';
            } 
            else {
                strcpy(resp.serialNo, "0000000000");
            }
            resp.hdr.mid = CM_GET_CHASSIS_SERIAL;
            chassisd_sendMsg(&resp, sizeof(resp), srcApp, srcCpu, srcSlot,
                             NULL, 0, 0);
            break;
        }

        case CM_GET_REVISION_MODEL:
        {
            cmMsgRevModelResp_t msg;
            seepPCB_t seep;

            CM_DEBUG(("cm_rcvMsg, CM_GET_REVISION_MODEL\n"));
            bzero(&msg, sizeof(cmMsgRevModelResp_t));

            /* The 4-digit model number is found after skipping 8 chars
             * of the official format of model string in the SEEP.
             * For example: ONS-SYS-2220
             */
            if (cm_read_seep_bc(seep.model_num,
                                offsetof(seepPCB_t, model_num[0]),
                                sizeof(seep.model_num)) == NFX_OK) {
                sprintf(msg.model, "ONStor %4s NAS Gateway",
                        &seep.model_num[8]);
            } else {
                sprintf(msg.model, "ONStor XXXX NAS Gateway");
            }
            msg.hdr.mid = CM_GET_REVISION_MODEL;
            chassisd_sendMsg(&msg, sizeof(msg), srcApp, srcCpu, srcSlot,
                             NULL, 0, 0);
            break;
        }
#endif
		default:
			printf("Invalid Message Received, mid = %d\n", mid);
	}

	return 0;

}

/*++

Routine Description:
    This routine sends a NODE event to the event manager

Arguments:
    reason - Reason for reboot.  String for autosupport messages.  

Return Value:
    none

--*/
void
cmSendNodeEvent(char *reason)
{
    nfx_eventMsg_t eventMsg;
	int rc;

    bzero(&eventMsg, sizeof(eventMsg));
    eventMsg.msgType = NFX_EVENT_POST;
    eventMsg.eventType = NFX_EVENT_NODE;
    eventMsg.eventScope = NFX_EVENT_LOCAL;
    eventMsg.needAck = FALSE;
    eventMsg.param.nodeEvent.state = EVENT_STATE_DOWN;
    snprintf(eventMsg.msg_to_gen_autosupport,
        sizeof(eventMsg.msg_to_gen_autosupport), reason);

    rc = chassisd_sendMsg(&eventMsg, sizeof(nfx_eventMsg_t),
					  "eventd", NFX_SSC_UNIX, SSC_SLOT, 0, 0, 0);
	if (rc < 0) {
        E_LOG(class_1, err_s, eee_getApplicationId("cm"), 0, 0, 0,
              ("%s: Error sending Node down event to eventd %d.\n",
              __FUNCTION__, rc));
	}

    autosupport_log_reboot_state(&eventMsg, 0);
}

/*-----------------------------------------------------------------
 * Name: chassisd_procRebootReq()
 *
 * Description: process the RebootReq message sent by CLI/GUI to chassisd
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procRebootReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgRebootResp_t	resp;
	chassisdMsgRebootReq_t	* req;
    nfx_eventMsg_t eventMsg;
	uint32	slot;
	uint32	cpu;
	int rc;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procRebootReq: Enter\n");
#endif


	/*
	 * Send Response back to requester
	 */
	resp.hdr.mid = CHASSISD_REBOOT_RESP;
	resp.rc = CM_REBOOT_OK;
	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);
	req = (chassisdMsgRebootReq_t *)(mdesc->msg);
	/*
	 * Send Reboot Request to toshiba
	 */
	chassisd_sendMsg(mdesc->msg, mdesc->len,
					"cm", NFX_SSC_MGMT, SSC_SLOT,
					NULL, 0, 0);
	/*
	 * Remove old version strings
	 */
	if (req->reboot_type == CM_REBOOT_TYPE_SLOT) {
		slot = req->slot;
   		bzero(&eventMsg, sizeof(eventMsg));
   		eventMsg.msgType = NFX_EVENT_POST;
   		eventMsg.eventType = NFX_EVENT_CARD;
   		eventMsg.eventScope = NFX_EVENT_LOCAL;
   		eventMsg.param.cardEvent.slot = slot;
   		eventMsg.param.cardEvent.cardType = cm_cardType[slot];
   		eventMsg.param.cardEvent.state = EVENT_STATE_DOWN;
   		rc = sendAgileMsg(chassisd_rxif, &eventMsg, sizeof(nfx_eventMsg_t), 
                          "eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, 0,0,0);
	    if (rc < 0) {
            E_LOG(class_1, err_s, eee_getApplicationId("cm"), 0, 0, 0,
                  ("CM: %s: Error sending agile msg to eventd %d.\n",
			      __FUNCTION__, rc));
	    }

   		E_LOG (class_1, notice_s, eee_getApplicationId("cm"), 0, 0, 0,
              ("CHASSISD: Reboot slot %d", slot));

		bzero(cm.slot[slot].bmfpga_ver, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].luc_ver, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].lmux_ver, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].hw_ver, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].hw_serialNo, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].chassis_serialNo, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].chassis_hw, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].chassis_macAddr, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].boot_dev, CM_VER_STRING_LEN);
		for (cpu = 0; cpu < CM_NUM_CPUS; cpu++) {
			bzero(cm.slot[slot].cpu[cpu].prom_ver, CM_VER_STRING_LEN);
			bzero(cm.slot[slot].cpu[cpu].rt_ver, CM_VER_STRING_LEN);
		}
	}
	else {
		slot = req->slot;
		cpu = req->slice;

		E_LOG (class_1, notice_s, eee_getApplicationId("cm"), 0, 0, 0,
                        ("CHASSISD: Reboot slot %d, cpu %d", slot, cpu));

		bzero(cm.slot[slot].cpu[cpu].prom_ver, CM_VER_STRING_LEN);
		bzero(cm.slot[slot].cpu[cpu].rt_ver, CM_VER_STRING_LEN);
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: chassisd_procInfoReq()
 *
 * Description: process the InfoReq message sent by CLI/GUI to chassisd
 *				Returns info stored in the cm_t structure.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procInfoReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgInfoResp_t	resp;
	int						slot, cpu;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procInfoReq: Enter\n");
#endif

	/*
	 * Send Response back to requester
	 */
	bzero(&resp, sizeof(resp));

	resp.hdr.mid = CHASSISD_GET_CHASSIS_INFO_RESP;
	resp.chassis_type = 0;						/* TODO */
	for (slot=0; slot<CM_NUM_SLOTS;slot++) {
		if (cm.slot[slot].present) {
			resp.present[slot] = 1;
			resp.num_cpus[slot] = cm.slot[slot].num_cpus;
			resp.board_type[slot] = cm.slot[slot].type;
			for (cpu=0; cpu<cm.slot[slot].num_cpus; cpu++) {
				resp.cpu_state[slot][cpu] = cm.slot[slot].cpu[cpu].state;
            }
		}
		else {
			resp.present[slot] = 0;
			resp.board_type[slot] = 0;
			resp.num_cpus[slot] = 0;
			for (cpu=0; cpu<CM_NUM_CPUS; cpu++)
				resp.cpu_state[slot][cpu] = CM_SLICE_STATE_NOT_PRESENT;
		}
	}
	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: chassisd_procVersionReq()
 *
 * Description: process the VersionReq message sent by CLI/GUI to chassisd
 *				Returns info stored in the cm_t structure.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procVersionReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgVersionResp_t	resp;
	int							slot, cpu;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procVersionReq: Enter\n");
#endif

	/*
	 * Send Response back to requester
	 */
	bzero(&resp, sizeof(resp));

	resp.hdr.mid = CHASSISD_GET_VERSION_RESP;
	for (slot=0; slot<CM_NUM_SLOTS;slot++) {
		if (cm.slot[slot].present) {
			resp.present[slot] = 1;
			resp.board_type[slot] = cm.slot[slot].type;
			memcpy(resp.bmfpga_ver[slot], cm.slot[slot].bmfpga_ver,
														CM_VER_STRING_LEN);
			memcpy(resp.luc_ver[slot], cm.slot[slot].luc_ver,
														CM_VER_STRING_LEN);
			memcpy(resp.lmux_ver[slot], cm.slot[slot].lmux_ver,
														CM_VER_STRING_LEN);
			memcpy(resp.hw_ver[slot], cm.slot[slot].hw_ver,
														CM_VER_STRING_LEN);
			memcpy(resp.hw_serialNo[slot], cm.slot[slot].hw_serialNo,
														CM_VER_STRING_LEN);
			memcpy(resp.chassis_serialNo[slot], cm.slot[slot].chassis_serialNo,
														CM_VER_STRING_LEN);
			memcpy(resp.chassis_hw[slot], cm.slot[slot].chassis_hw,
														CM_VER_STRING_LEN);
			memcpy(resp.chassis_macAddr[slot], cm.slot[slot].chassis_macAddr,
														CM_VER_STRING_LEN);
			memcpy(resp.boot_dev[slot], cm.slot[slot].boot_dev,
														CM_VER_STRING_LEN);
//			for (cpu=0; cpu<cm.slot[slot].num_cpus; cpu++) {
			for (cpu=0; cpu<CM_NUM_CPUS; cpu++) {
				memcpy(resp.prom_ver[slot][cpu],cm.slot[slot].cpu[cpu].prom_ver,
														CM_VER_STRING_LEN);
				memcpy(resp.rt_ver[slot][cpu], cm.slot[slot].cpu[cpu].rt_ver,
														CM_VER_STRING_LEN);
			}
		}
	}
	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: chassisd_procPowerReq()
 *
 * Description: process the PowerReq message sent by CLI/GUI to chassisd
 *				Returns info stored in the cm_t structure.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procPowerReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgPowerResp_t	resp;
	int						slot;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procPowerReq: Enter\n");
#endif

	/*
	 * Send Response back to requester
	 */
	bzero(&resp, sizeof(resp));

	resp.hdr.mid = CHASSISD_GET_POWER_RESP;
	for (slot=0; slot<CM_NUM_SLOTS;slot++) {
		if (cm.slot[slot].present) {
			resp.present[slot] = 1;
			resp.valid[slot] = cm.slot[slot].power_valid;
			resp.power[slot] = cm.slot[slot].power;
		}
	}
	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: chassisd_procTempReq()
 *
 * Description: process the TempReq message sent by CLI/GUI to chassisd
 *				Returns info stored in the cm_t structure.
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procTempReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgTempResp_t	resp;
	int						slot;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procTempReq: Enter\n");
#endif

	/*
	 * Send Response back to requester
	 */
	bzero(&resp, sizeof(resp));

	resp.hdr.mid = CHASSISD_GET_TEMP_RESP;
	for (slot=0; slot<CM_NUM_SLOTS;slot++) {
		if (cm.slot[slot].present) {
			resp.present[slot] = 1;
			resp.valid[slot] = cm.slot[slot].temp_valid;
			resp.temp[slot] = cm.slot[slot].temp;
		}
	}
	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:	cm_procVersionReq()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
int
cm_procVersionReq(
	chassisd_msgDesc_t	* mdesc
)
{
	cmMsgVersionReq_t	* buf = (cmMsgVersionReq_t *)(mdesc->msg);

#if CHASSISD_DEBUG_RX_MSG
	printf("cm_procVersionReq: Enter\n");
#endif

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/23/02
 *
 *-----------------------------------------------------------------
 */
int
cm_procInfoResp(
	chassisd_msgDesc_t	* mdesc
)
{
	cmMsgInfoResp_t	* buf = (cmMsgInfoResp_t *)(mdesc->msg);
	int				slot, cpu;

#if CHASSISD_DEBUG_RX_MSG
	printf("cm_procInfoResp: Enter\n");
#endif

	for (slot=0;slot<CM_NUM_SLOTS;slot++) {
		cm.slot[slot].present = buf->present[slot];
		cm.slot[slot].type = buf->board_type[slot];
		cm.slot[slot].temp = buf->temp[slot];
		cm.slot[slot].temp_valid = buf->temp_valid[slot];
		cm.slot[slot].power = buf->power[slot];
		cm.slot[slot].power_valid = buf->power_valid[slot];
		cm.slot[slot].num_cpus = buf->num_cpus[slot];
		for (cpu=0; cpu<buf->num_cpus[slot]; cpu++)
			cm.slot[slot].cpu[cpu].state = buf->cpu_state[slot][cpu];
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: cm_procVersionResp()
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/21/02
 *
 *-----------------------------------------------------------------
 */
int
cm_procVersionResp(
	chassisd_msgDesc_t	* mdesc
)
{
	cmMsgVersionResp_t	* buf = (cmMsgVersionResp_t *)(mdesc->msg);
	cm_msg_hdr_t		* hdr = (cm_msg_hdr_t *)buf;
    int                   len = CM_VER_STRING_LEN - 1;

#if CHASSISD_DEBUG_RX_MSG
	printf("cm_procVersionResp: Enter\n");
#endif

	if (buf->which_ver == CM_VER_STRING_PROM) {
		strncpy(cm.slot[hdr->srcSlot].cpu[hdr->srcCpu].prom_ver, buf->ver, len);
        cm.slot[hdr->srcSlot].cpu[hdr->srcCpu].prom_ver[len] = '\0';
	}
	else if (buf->which_ver == CM_VER_STRING_RT) {
		strncpy(cm.slot[hdr->srcSlot].cpu[hdr->srcCpu].rt_ver, buf->ver, len);
        cm.slot[hdr->srcSlot].cpu[hdr->srcCpu].rt_ver[len] = '\0';
	}
	else if (buf->which_ver == CM_VER_STRING_BMFPGA) {
		strncpy(cm.slot[hdr->srcSlot].bmfpga_ver, buf->ver, len);
        cm.slot[hdr->srcSlot].bmfpga_ver[len] = '\0';
	}
	else if (buf->which_ver == CM_VER_STRING_LUC) {
		strncpy(cm.slot[hdr->srcSlot].luc_ver, buf->ver, len);
        cm.slot[hdr->srcSlot].luc_ver[len] = '\0';
	}
	else if (buf->which_ver == CM_VER_STRING_LMUX) {
		strncpy(cm.slot[hdr->srcSlot].lmux_ver, buf->ver, len);
        cm.slot[hdr->srcSlot].lmux_ver[len] = '\0';
	}
	else if (buf->which_ver == CM_VER_STRING_HW) {
		strncpy(cm.slot[hdr->srcSlot].hw_ver, buf->ver, len);
        cm.slot[hdr->srcSlot].hw_ver[len] = '\0';
	}
	else if (buf->which_ver == CM_VER_STRING_SERIALNO) {
		strncpy(cm.slot[hdr->srcSlot].hw_serialNo, buf->ver, len);
        cm.slot[hdr->srcSlot].hw_serialNo[len] = '\0';
	}
	else if (buf->which_ver == CM_VER_STRING_CHASSIS_SERIALNO) {
		strncpy(cm.slot[hdr->srcSlot].chassis_serialNo, buf->ver, len);
        cm.slot[hdr->srcSlot].chassis_serialNo[len] = '\0';
	}
	else if (buf->which_ver == CM_VER_STRING_CHASSIS_HW) {
		strncpy(cm.slot[hdr->srcSlot].chassis_hw, buf->ver, len);
        cm.slot[hdr->srcSlot].chassis_hw[len] = '\0';
	}
	else if (buf->which_ver == CM_VER_STRING_CHASSIS_MACADDR) {
		strncpy(cm.slot[hdr->srcSlot].chassis_macAddr, buf->ver, len);
        cm.slot[hdr->srcSlot].chassis_macAddr[len] = '\0';
	}
	else if (buf->which_ver == CM_VER_STRING_PRIMARY_BOOTDEV) {
		strncpy(cm.slot[hdr->srcSlot].boot_dev, buf->ver, len);
        cm.slot[hdr->srcSlot].boot_dev[len] = '\0';
	}

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
int
chassisd_procDebugReq(
	chassisd_msgDesc_t	* mdesc
)
{
	chassisdMsgDebugResp_t	resp;

#if CHASSISD_DEBUG_RX_MSG
	printf("chassisd_procDebugReq: Enter\n");
#endif

	/*
	 * Send Response back to requester
	 */
	bzero(&resp, sizeof(resp));

	resp.hdr.mid = CHASSISD_DEBUG_RESP;

	memcpy(&(resp.cm), &cm, sizeof(cm_t));

	chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					NULL, 0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: cm_procHeartbeatInfo
 *
 * Description: When Toshiba sends the chassis info, 
 *              send it along to TXRX.
 *
 * Created by:	lipika
 *
 * Date Created:	6/12/03
 *
 *-----------------------------------------------------------------
 */
int
cm_procHeartbeatInfo(
	chassisd_msgDesc_t	* mdesc
)
{
	cmHeartbeatMsgInfo_t req, *msgPtr;
	cmHeartbeatAck_t ack;

    /* check if toshiba is requesting system reboot
     */
    msgPtr = (cmHeartbeatMsgInfo_t *)(mdesc->msg);
    cm.slot[NFX_SLOTTYPE_SSC].cpu[NFX_SSC_MGMT].state = 
            msgPtr->cpu_state[NFX_SLOTTYPE_SSC][NFX_SSC_MGMT];
    if (msgPtr->cpu_state[NFX_SLOTTYPE_SSC][NFX_SSC_MGMT] == 
        CM_SLICE_STATE_RESET_WAIT)
    {
        utils_reboot_system("Node going down for reboot! "
		"(chassisd: ssc-mgmt requested system reboot).", "cm", 0);
    }
    /* mark that heartbeat was received
     */
    cm_heartbeat_rx += 1;

	/* Forward msg to TXRX */
	req.hdr.mid = CM_SEND_HEARTBEAT_INFO;
	req.hdr.srcSlot = 0;
	req.hdr.srcCpu = 0;
	req.hdr.srcApp = chassisd_rxif;
	req.hdr.srcPlane = MGMTPLANE;

	(void) chassisd_sendMsg(&req, sizeof(req), "cm",
							0, 1, NULL,
							0, 0);

	/* Send ACK to Toshiba */
	ack.hdr.mid = CM_SEND_HEARTBEAT_ACK;
	ack.hdr.srcSlot = 0;
	ack.hdr.srcCpu = 0;
	ack.hdr.srcApp = chassisd_rxif;
	ack.hdr.srcPlane = MGMTPLANE;

	(void) chassisd_sendMsg(&ack, sizeof(ack), "cm",
							1, 0, NULL,
							0, 0);

	return NFX_OK;
}

/*-----------------------------------------------------------------
 * Name: convert_to_magicNo
 *
 * Description: Converts the given string to a number
 *
 * Created by:	Lipika Bhattacharya
 *
 * Date Created:	9/18/03
 *
 *-----------------------------------------------------------------
 */
int
convert_to_magicNo(char* str)
{
	char orig_str[11];
	int i=0;
	int magicNo=0;
	
	strcpy((char *)&orig_str, str);
	for(i=0;i<10;i++) {
		if((orig_str[i] >= '0') && (orig_str[i] != '\0'))
		 	magicNo += orig_str[i] & 0x55;	
	}	
	return(magicNo);
}

/*-----------------------------------------------------------------
 * Name: chassis_send_memoryUpgrade
 *
 * Description: Sends command to the SSC_MGMT to 
 *              update the model num in the chassis seep.
 *
 * Created by:	Lipika Bhattacharya
 *
 * Date Created:	9/18/03
 *
 *-----------------------------------------------------------------
 */
int
chassis_send_memoryUpgrade(chassisd_msgDesc_t *mdesc)
{
	int		rc;
	chassisdMsgMemUpgradeReq_t *msg = (chassisdMsgMemUpgradeReq_t *)(mdesc->msg);
	cmMsgMemUpgradeReq_t req;
	cmMsgHeartbeatControlReq_t hb_req;
	cmMsgSerialNoResp_t	resp_from_txrx;
	cmMsgMemUpgradeAck_t	ack_from_txrxfp;
//	cmMsgMemUpgradeAck_t	ack_from_sscm;
	chassisdMsgMemUpgradeResp_t resp;
	struct timeval			to;
	int systemId=0;
	static int txrx_ack=0;
	static int fp_ack=0;
	int from_system=0;

	to.tv_sec = 10;
	to.tv_usec = 0;

	bzero(&resp_from_txrx, sizeof(resp_from_txrx));
	bzero(&ack_from_txrxfp, sizeof(ack_from_txrxfp));
	bzero(&resp, sizeof(resp));
	txrx_ack = 0;
	fp_ack = 0;

	/* Copy the user input into local variable */
	systemId = msg->userId;

	/* Request for TXRX serial id */
	bzero(&req, sizeof(req));
	req.hdr.mid = CHASSISD_GET_SERIALNUM;
	req.hdr.srcSlot = 0;
	req.hdr.srcCpu = 0;
	req.hdr.srcApp = chassisd_rxif;
	req.hdr.srcPlane = MGMTPLANE;

	rc = chassisd_sendMsg(&req, sizeof(req), "cm", 0, 1, /* txrx */
						&to,
						&resp_from_txrx, sizeof(resp_from_txrx));

	if(rc > 0) {
	    if(resp_from_txrx.hdr.mid != CM_SET_SERIALNO_RESP) 
	    {
		    resp.hdr.mid = CHASSISD_MEM_UPGRADE_RESP;
		    resp.result = MEMUPGRADE_ERR_TXRX_MID_MISMATCH;
	            chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					&to, 0, 0);
	        return 0;
	    }	
	    else {
		from_system = convert_to_magicNo((char *)&resp_from_txrx.serialNo);
		//printf("from_serial %d %x \n", from_system, from_system);
		//printf("systemId %d %x \n", systemId,systemId );
		if(from_system != systemId) {
		    resp.hdr.mid = CHASSISD_MEM_UPGRADE_RESP;
		    resp.result = MEMUPGRADE_ERR_SYSTEM_ID_MISMATCH;
	            chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					&to, 0, 0);
		    return 0;
	        }
		else { /* comparison passed */

			/* First turn off the heartbeat feature on the TXRX & SSCM
			 * to prevent confusion in recieving response
			 */  
			bzero(&hb_req, sizeof(hb_req));
			hb_req.hdr.srcSlot = 0;
			hb_req.hdr.srcCpu = 0;
			hb_req.hdr.srcApp = chassisd_rxif;
			hb_req.hdr.srcPlane = MGMTPLANE;
			hb_req.hdr.mid = CM_SET_HEARTBEAT_OFF;
			hb_req.feature_en = 0;
			chassisd_sendMsg(&hb_req, sizeof(hb_req), "cm", 0, 1, /* txrx */
								&to,
								0,0);

			chassisd_sendMsg(&hb_req, sizeof(hb_req), "cm", 1, 0, /* toshiba */
								&to,
								0,0);

			/* based on string passed in set upgrade_type */
			/* For now there is only one, so just set */
			bzero(&req, sizeof(req));
			req.hdr.mid = CHASSISD_SET_MEM_UPGRADE;
			req.hdr.srcSlot = 0;
			req.hdr.srcCpu = 0;
			req.hdr.srcApp = chassisd_rxif;
			req.hdr.srcPlane = MGMTPLANE;
			req.memUpgrade_type = CM_MEMUPGRADE_TO_8G;

			/* Send Seep update msg to TXRX */
			bzero(&ack_from_txrxfp, sizeof(ack_from_txrxfp));
			rc = chassisd_sendMsg(&req, sizeof(req), "cm", 0, 1, /* txrx */
								&to,
								&ack_from_txrxfp,sizeof(ack_from_txrxfp));

			/* Save response form TXRX */
			if(ack_from_txrxfp.hdr.mid == CM_SET_MEM_UPGRADE_ACK) {
				txrx_ack = ack_from_txrxfp.ack;
			}

			/* Send Seep update msg to FP */
			bzero(&ack_from_txrxfp, sizeof(ack_from_txrxfp));
			chassisd_sendMsg(&req, sizeof(req), "cm", 2, 1,
								&to,
								&ack_from_txrxfp,sizeof(ack_from_txrxfp));

			/* Save response form FP */
			if(ack_from_txrxfp.hdr.mid == CM_SET_MEM_UPGRADE_ACK) 
				fp_ack = ack_from_txrxfp.ack;

			/* Send system update msg to Toshiba ONLY if TXRX & FP is ok */
			if((txrx_ack == MEMUPGRADE_ACK_OK) && (fp_ack == MEMUPGRADE_ACK_OK)) {
				//bzero(&ack_from_sscm, sizeof(ack_from_sscm));
				chassisd_sendMsg(&req, sizeof(req), "cm", 1, 0, /* toshiba */
									&to,
									0,0);
				//if(ack_from_sscm.hdr.mid == CM_SET_MEM_UPGRADE_ACK) 
				//	sscm_ack = ack_from_sscm.ack;
			}	
			else {
				/* If upgrade failed, we need to turn on heartbeat again */
				bzero(&hb_req, sizeof(hb_req));
				hb_req.hdr.srcSlot = 0;
				hb_req.hdr.srcCpu = 0;
				hb_req.hdr.srcApp = chassisd_rxif;
				hb_req.hdr.srcPlane = MGMTPLANE;
				hb_req.hdr.mid = CM_SET_HEARTBEAT_OFF;
				hb_req.feature_en = 1;
				chassisd_sendMsg(&hb_req, sizeof(hb_req), "cm", 0, 1, /* txrx */
									&to,
									0,0);

				chassisd_sendMsg(&hb_req, sizeof(hb_req), "cm", 1, 0, /* toshiba */
									&to,
									0,0);
			}	
			bzero(&resp, sizeof(resp));
			resp.hdr.mid = CHASSISD_MEM_UPGRADE_RESP;
			resp.txrxAck = txrx_ack;
			resp.fpAck = fp_ack;
			resp.result = MEMUPGRADE_ACK_OK;
	       		chassisd_sendMsg(&resp, sizeof(resp),
						mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
						NULL, 0, 0);

		}	
	    }	    
	}
	else {
		    resp.hdr.mid = CHASSISD_MEM_UPGRADE_RESP;
		    resp.result = MEMUPGRADE_ERR_TXRX_NO_RCV_RESP;
	            chassisd_sendMsg(&resp, sizeof(resp),
					mdesc->srcApp, mdesc->srcCpu, mdesc->srcSlot,
					&to, 0, 0);
	        return 0;
	}	
	return rc;
}
#endif	/* SSC */
