/*
 *-----------------------------------------------------------------
 *
 * Name:
 *
 * RCS:  $Id: chassisd.c,v 1.4 2002/04/30 20:46:47 rick Exp $
 *
 * Copyright (C) 2002, Agile Storage, Inc.
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/18/02
 *
 *-----------------------------------------------------------------
 */
#ifdef SSC

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include "../sm-anpssc/anpssc-api.h"
#include "nfx-incl.h"
#include "../sm-elog/elog-api.h"
#include "../sm-event/event-api.h"
#include "../sm-seep/seep.h"
#include "cm.h"
#include "cm-api.h"
#if defined(COUGAR)
#include "cm-api-cg.h"
#include "../ssc-genlib/cm-util-bc.h"
#elif defined(BOBCAT)
#include "cm-api-bc.h"
#include "../ssc-genlib/cm-util-bc.h"
#endif
#include "../ssc-genlib/genlib.h"
#ifdef DMALLOC
#include "dmalloc.h"
#endif

/* SSC heartbeat timeout in seconds
 */
#define CM_HEARTBEAT_TIMEOUT 15

cm_t	cm;
extern int cm_heartbeat_rx;
extern int chassisd_rxif;

/* Local Declarations
 */
static void     chassisd_sigTerm(int sig);
static void     chassisd_sigHup(int sig);

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/21/02
 *
 *-----------------------------------------------------------------
 */
int
main(void)
{
	int	slot, cpu;
	sigset_t set;
#ifdef CHEETAH
	int	rc;
    struct timeval tv, tv_timer, to;
    cmMsgInfoReq_t req;
    cmMsgInfoResp_t resp;
#endif

	//-- Setup the signal handling.
	sigfillset(&set);
	sigdelset(&set, SIGALRM);
	sigdelset(&set, SIGTERM);
	sigdelset(&set, SIGHUP);
	sigdelset(&set, SIGCHLD);
	sigdelset(&set, SIGSEGV);
	sigdelset(&set, SIGBUS);
    sigdelset(&set, SIGILL);
    sigdelset(&set, SIGSYS);
    sigdelset(&set, SIGTRAP);
	sigprocmask(SIG_SETMASK, &set, NULL);

	//-- Set up our termination handler
	signal (SIGTERM, chassisd_sigTerm);
	signal (SIGHUP, chassisd_sigHup);

	bzero(&cm, sizeof(cm_t));

	chassisd_msgInit();
#ifdef CHEETAH
	/*
	 * Get current state of the chassis
	 */
	cm_sendInfoReq();
	/*
	 * Get version strings from everyone
	 */
	for (slot=0; slot<CM_NUM_SLOTS; slot++) {
#else
	for (slot=1; slot<CM_NUM_SLOTS; slot++) {
#endif
		for (cpu=0; cpu<CM_NUM_CPUS; cpu++) {
			if ((slot == 0) && (cpu == 0)) {
				/* do nothing */
			} else {
				cm_sendVersionReq(slot, cpu);
            }
		}
	}

	kill (getppid(), SIGUSR1);
	E_LOG (class_1, notice_s, eee_getApplicationId("cm"), 0, 0, 0,
						("CHASSISD: Chassis Manager: Started"));
#if defined(BOBCAT) || defined(COUGAR)
    chassisd_run();
#else
    to.tv_sec = 5;
    to.tv_usec = 0;
    req.hdr.mid = CM_SET_INFO_REQ;
    req.hdr.srcSlot = 0;
    req.hdr.srcCpu = 0;
    req.hdr.srcApp = chassisd_rxif;
    req.hdr.srcPlane = MGMTPLANE;
    if (gettimeofday(&tv, NULL) == 0) {
        tv_timer = tv;
    }
    else {
        E_LOG (class_1, err_s, eee_getApplicationId("cm"), 0, 0, 0,
              ("CHASSISD: Chassis Manager: couldn't get time of day."));
        exit(0);
    }

    while (1) {
        chassisd_rcvMsg();
        if (gettimeofday(&tv, NULL) == 0) {
            if (tv_timer.tv_sec + CM_HEARTBEAT_TIMEOUT < tv.tv_sec) {
                cmSetWatchDog(CM_WATCHDOG_TIMEOUT);
                tv_timer = tv;
                /* plan to reboot system if no heartbeat was received
                 */
                if (cm_heartbeat_rx != 0) {
                    cm_heartbeat_rx = 0;
                }
                else {
                    /* try to contact toshiba one more time and
                     * reboot if failed
                     */
                    rc = sendAgileMsg(chassisd_rxif, (char *)&req, sizeof(req),
                                      "cm", 1, 0, 0, &to, &resp, sizeof(resp));
                    if (rc < 0) {
                        E_LOG(class_1, err_s, eee_getApplicationId("cm"),0,0,0,
                        ("CHASSISD: Chassis Manager: couldn't send msg to cm"));
                        exit(0);
                    }
                    if (rc != sizeof(resp)) {
                        utils_reboot_system("Node going down for reboot! "
							"(chassisd: because ssc-mgmt heartbeat is down).",
							"cm", 0);
                    }
                }
            }
            /* if system time was changed to go backwards then need to resync
             */ 
            else 
            if (tv_timer.tv_sec > tv.tv_sec) {
                tv_timer = tv;
                cmSetWatchDog(CM_WATCHDOG_TIMEOUT);
            }
        }
        else {
            E_LOG (class_1, err_s, eee_getApplicationId("cm"), 0, 0, 0,
                  ("CHASSISD: Chassis Manager: couldn't get time of day."));
            exit(0);
        }
    }
#endif
    return 0;
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
static void
chassisd_sigTerm(int sig)
{
    cmDisableWatchDog();
	E_LOG (class_1, notice_s, eee_getApplicationId("cm"), 0, 0, 0,
				("CHASSISD: Chassis Manager: Terminated"));
	exit(0);
}

/*-----------------------------------------------------------------
 * Name:
 *
 * Description:
 *
 * Created by:	Rick Lund
 *
 * Date Created:	4/25/02
 *
 *-----------------------------------------------------------------
 */
static void
chassisd_sigHup(int sig)
{
	E_LOG (class_1, notice_s, eee_getApplicationId("cm"), 0, 0, 0,
				("CHASSISD: Chassis Manager: Re-Started"));
	exit(0);
}

/*++

Routine Description:
    This routine sends a temperature event to the event manager

Arguments:
    temp  - temperature in celsius
    state - operational state (EVENT_STATE_UP, EVENT_STATE_DOWN)

Return Value:
    none

--*/
void
cmSendTemperatureEvent(uint32 temp, int state)
{
    nfx_eventMsg_t eventMsg;
	int rc;

    bzero(&eventMsg, sizeof(eventMsg));
    eventMsg.msgType = NFX_EVENT_POST;
    eventMsg.needAck = FALSE;
    eventMsg.eventScope = NFX_EVENT_LOCAL;
    eventMsg.eventType = NFX_EVENT_TEMPERATURE;
    if (state == EVENT_STATE_UP)
        snprintf(eventMsg.msg_to_gen_autosupport, EVENT_MAX_MAIL_SUB-1,
                 "Intake air temperature reduced to %d degrees Celsius.\n",
                 temp);
    else
        snprintf(eventMsg.msg_to_gen_autosupport, EVENT_MAX_MAIL_SUB-1,
                 "Intake air temperature at %d degrees Celsius. "
                 "Check external temperature and airflow.\n", temp);

    rc = sendAgileMsg(chassisd_rxif, &eventMsg, sizeof(nfx_eventMsg_t),
					  "eventd", NFX_SSC_UNIX, SSC_SLOT, MGMTPLANE, 0,0,0);
	if (rc < 0) {
        E_LOG(class_1, err_s, eee_getApplicationId("cm"), 0, 0, 0,
              ("CM: %s: Error sending agile msg to eventd %d.\n",
			  __FUNCTION__, rc));
	}
}

/*++

Routine Description:
    Reset the MAC device for FP ports.  This function is called
    after a crash to prevent flooding of pause frames.

Arguments:
    None

Return Value:
    None

--*/
void
cmResetMacs(void)
{
    uint32 gpp;

    if (mm_readAddr(&gpp, sizeof(gpp), CM_BMFPGA_GPP) == NFX_OK) {
        gpp |= CM_GIGE_RESET_BITS;
        if (mm_writeAddr(&gpp, sizeof(gpp), CM_BMFPGA_GPP) != NFX_OK) {
            E_LOG(class_1, warning_s, eee_getApplicationId("cm"), 0, 0, 0,
                  ("CM: Couldn't update BM_FPGA GPP.\n"));
        }
    } else {
        E_LOG(class_1, warning_s, eee_getApplicationId("cm"), 0, 0, 0,
              ("CM: Couldn't read BM_FPGA GPP.\n"));
    }
}

#ifndef CHEETAH
/*++

Routine Description:
    This routine checks the intake temperature status and sends alarms if
    necessary.  When temperature has risen above max threshold, an event
    message is sent as a notification of the error.  Otherwise, if the
    the temperature has remained above the max threshold for TEMP_ERR_COUNT
    times in a row or has gone higher, an ELOG message is periodically sent.
    When temperature subsides at least 3 degrees below the max threshold,
    an event message is sent as a notification of a recovery.

Arguments:
    none

Return Value:
    NFX_OK or NFX_ERR

--*/
int
cm_checkTempStatus(void)
{
    static uint32 high_temp = 0;
    static uint32 temp_err_count = 0;
    uint32 temp;

    if (cm_read_temperature_bc(&temp) == NFX_OK) {
        if (temp >= FAN_MAX_CHASSIS_TEMP) {
            if (!temp_err_count) {
                cmSendTemperatureEvent(INTAKE_TEMP(temp), EVENT_STATE_DOWN);
            } else if (!(temp_err_count++ % TEMP_ERR_COUNT) ||
                       (temp > high_temp)) {
                E_LOG(class_1, warning_s, eee_getApplicationId("cm"), 0, 0, 0,
                      ("CM: Intake air temperature at %d degrees Celsius.  "
                       "Check external temperature and airflow.\n",
                      INTAKE_TEMP(temp)));
            }
        } else if ((temp_err_count) && (temp <= (FAN_MAX_CHASSIS_TEMP-3))) {
            cmSendTemperatureEvent(INTAKE_TEMP(temp), EVENT_STATE_UP);
            temp_err_count = high_temp = 0;
        }

        if (temp > high_temp) {
            high_temp = temp;
        }

        return NFX_OK;
    }

    return NFX_ERR;
}

/*++

Routine Description:
    This routine gets the SSC PROM version string.

Arguments:
    verStr - pointer to version string returned
             string will be null terminated, and there should be space for 
             at least CM_VER_STRING_LEN chars including null terminator

Return Value:
    None

--*/
void
cmGetVersionSSC(uchar8 *verStr)
{
    uchar8 ver[CM_VER_STRING_LEN];

    if (cm_read_seep_bc((void *)ver, SEEP_NVRAM_START, 
                        SEEP_VER_STRING_LEN_SIZE) == NFX_OK) {
        strncpy(verStr, ver, SEEP_VER_STRING_LEN_SIZE);
        if (SEEP_VER_STRING_LEN_SIZE < CM_VER_STRING_LEN) {
            verStr[SEEP_VER_STRING_LEN_SIZE] = '\0';
        }
        else {
            verStr[CM_VER_STRING_LEN-1] = '\0';
        }
    }
    else {
        sprintf(verStr, "PROM_SSC: unknown");
    }
    return;
}

/*++

Routine Description:
    This routine checks the boot device and sets the compact flash LED that
    was used (and still mounted) to amber.  The other compact flash LED of
    the other device is set to green.  When a network boot occurs, the LED
    of both compact flashes are set to green.

Arguments
    none

Return Value
    none

--*/
void
cm_update_CFLED(void)
{
    int cf;
    uchar8 ver[CM_VER_STRING_LEN];

    cmGetBootDevInfo_bc(&ver[0], &cf);
    if ((cf == CM_CF_DEV1) || (cf == CM_CF_DEV2)) {
        cm_set_CFLED(cf, CM_LED_AMBER);
        if (cf == CM_CF_DEV1) {
            cm_set_CFLED(CM_CF_DEV2, CM_LED_GREEN);
        } else {
            cm_set_CFLED(CM_CF_DEV1, CM_LED_GREEN);
        }
    } else {
        cm_set_CFLED(CM_CF_DEV1, CM_LED_GREEN);
        cm_set_CFLED(CM_CF_DEV2, CM_LED_GREEN);
    }
}
#endif

#endif	/* SSC */

